@@ -30,12 +30,15 @@ class CustomModelConverter(ModelConverter):
         return field.name, BooleanSelectField(**kwargs)
     
     def handle_foreign_key(self, model, field, **kwargs):
-        if field.descriptor in (self.model_admin.raw_id_fields or ()):
+        if field.null:
+            kwargs['allow_blank'] = True
+        
+        if field.name in (self.model_admin.raw_id_fields or ()):
             # use a different widget here
             form_field = ModelSelectField(model=field.to, **kwargs)
         else:
             form_field = ModelSelectField(model=field.to, **kwargs)
-        return field.descriptor, form_field
+        return field.name, form_field
 
 
 class ModelAdmin(object):
@@ -44,10 +47,14 @@ class ModelAdmin(object):
     """
     paginate_by = 20
     columns = None
-    ignore_filters = ('ordering', 'page',)
+
     exclude_filter_fields = None
-    related_filters = []
+    ignore_filters = ('ordering', 'page',)
     raw_id_fields = None
+    related_filters = []
+    
+    delete_collect_objects = True
+    delete_recursive = True
     
     def __init__(self, admin, model):
         self.admin = admin
@@ -189,28 +196,54 @@ class ModelAdmin(object):
         
         return render_template('admin/models/edit.html', model_admin=self, instance=instance, form=form)
     
+    def collect_objects(self, obj):
+        select_queries, nullable_queries = obj.collect_queries()
+        objects = []
+        
+        for query, fk_field, depth in select_queries:
+            model = query.model
+            query.query = ['*']
+            collected = [obj for obj in query.execute().iterator()]
+            if collected:
+                objects.append((depth, model, fk_field, collected))
+        
+        return sorted(objects, key=lambda i: (i[0], i[1].__name__))
+    
     def delete(self):
         if request.method == 'GET':
             id_list = request.args.getlist('id')
-            query = self.model.select().where(**{
-                '%s__in' % self.model._meta.pk_name: id_list
-            })
-        elif request.method == 'POST':
+        else:
             id_list = request.form.getlist('id')
-            query = self.model.delete().where(**{
-                '%s__in' % self.model._meta.pk_name: id_list
-            })
-            results = query.execute()
-            flash('Successfully deleted %s %ss' % (results, self.get_display_name()), 'success')
+        
+        query = self.model.select().where(**{
+            '%s__in' % self.model._meta.pk_name: id_list
+        })
+        
+        if request.method == 'GET':
+            collected = {}
+            if self.delete_collect_objects:
+                for obj in query:
+                    collected[obj.get_pk()] = self.collect_objects(obj)
+        
+        elif request.method == 'POST':
+            count = query.count()
+            for obj in query:
+                obj.delete_instance(recursive=self.delete_recursive)
+            
+            flash('Successfully deleted %s %ss' % (count, self.get_display_name()), 'success')
             return redirect(url_for(self.get_url_name('index')))
         
-        return render_template('admin/models/delete.html', model_admin=self, query=query)
+        return render_template('admin/models/delete.html', **dict(
+            model_admin=self,
+            query=query,
+            collected=collected,
+        ))
     
     def collect_related_fields(self, model, accum, path):
         path_str = '__'.join(path)
         for field in model._meta.get_fields():
             if isinstance(field, ForeignKeyField):
-                self.collect_related_fields(field.to, accum, path + [field.descriptor])
+                self.collect_related_fields(field.to, accum, path + [field.name])
             elif model != self.model:
                 accum.setdefault((model, path_str), [])
                 accum[(model, path_str)].append(field)
@@ -232,7 +265,8 @@ class ModelAdmin(object):
         related = self.collect_related_fields(self.model, {}, [])
         
         if request.method == 'POST':
-            export = Export(filtered_query, related, request.form.getlist('fields'))
+            raw_fields = request.form.getlist('fields')
+            export = Export(filtered_query, related, raw_fields)
             return export.json_response()
         
         return render_template('admin/models/export.html',
@@ -310,6 +344,17 @@ class AdminTemplateHelper(object):
     def get_model_admins(self):
         return {'model_admins': self.admin.get_model_admins()}
     
+    def get_admin_url(self, obj):
+        model_admin = self.admin.get_admin_for(type(obj))
+        if model_admin:
+            return url_for(model_admin.get_url_name('edit'), pk=obj.get_pk())
+    
+    def get_model_name(self, model_class):
+        model_admin = self.admin.get_admin_for(model_class)
+        if model_admin:
+            return model_admin.get_display_name()
+        return model_class.__name__
+    
     def prepare_environment(self):
         self.app.template_context_processors[None].append(self.get_model_admins)
         
@@ -318,6 +363,8 @@ class AdminTemplateHelper(object):
         self.app.jinja_env.globals['get_verbose_name'] = self.get_verbose_name
         self.app.jinja_env.filters['fix_underscores'] = self.fix_underscores
         self.app.jinja_env.globals['update_querystring'] = self.update_querystring
+        self.app.jinja_env.globals['get_admin_url'] = self.get_admin_url
+        self.app.jinja_env.globals['get_model_name'] = self.get_model_name
 
 
 class Admin(object):
@@ -326,6 +373,7 @@ class Admin(object):
         self.app = app
         self.auth = auth
         
+        self._admin_models = {}
         self._registry = {}
         self._panels = {}
         
@@ -362,7 +410,7 @@ class Admin(object):
         model_admin = admin_class(self, model)
         admin_name = model_admin.get_admin_name()
         
-        self._registry[admin_name] = model_admin
+        self._registry[model] = model_admin
     
     def unregister(self, model):
         del(self._registry[model])
@@ -374,6 +422,9 @@ class Admin(object):
     def unregister_panel(self, title):
         del(self._panels[title])
     
+    def get_admin_for(self, model):
+        return self._registry.get(model)
+    
     def get_model_admins(self):
         return sorted(self._registry.values(), key=lambda o: o.get_admin_name())
     
@@ -436,11 +487,12 @@ class Export(object):
         self.query = query
         self.related = related
         self.fields = fields
+        
+        self.alias_to_model = dict([(k[1], k[0]) for k in self.related.keys()])
     
     def prepare_query(self):
         clone = self.query.clone()
         
-        alias_to_model = dict([(k[1], k[0]) for k in self.related.keys()])
         select = {}
         
         def ensure_join(query, m, p):
@@ -449,25 +501,25 @@ class Export(object):
                     next_model = query.model
                 else:
                     next, _ = p.rsplit('__', 1)
-                    next_model = alias_to_model[next]
+                    next_model = self.alias_to_model[next]
                     query = ensure_join(query, next_model, next)
                 
                 return query.switch(next_model).join(m)
             else:
                 return query
         
-        for field in self.fields:
-            # field may be something like "content" or "user__user_name"
-            if '__' in field:
-                path, column = field.rsplit('__', 1)
-                model = alias_to_model[path]
+        for lookup in self.fields:
+            # lookup may be something like "content" or "user__user_name"
+            if '__' in lookup:
+                path, column = lookup.rsplit('__', 1)
+                model = self.alias_to_model[path]
                 clone = ensure_join(clone, model, path)
             else:
                 model = self.query.model
-                column = field
+                column = lookup
             
             select.setdefault(model, [])
-            select[model].append((column, field))
+            select[model].append(column)
 
         clone.query = select
         return clone
@@ -481,7 +533,7 @@ class Export(object):
             yield '[\n'
             for obj in prepared_query:
                 i -= 1
-                yield json.dumps(serializer.serialize_object(obj, self.fields))
+                yield json.dumps(serializer.serialize_object(obj, prepared_query.query))
                 if i > 0:
                     yield ',\n'
             yield '\n]'
