@@ -89,19 +89,42 @@ class AdminAuthentication(UserAuthentication):
 
 class RestResource(object):
     paginate_by = 20
+    
+    # dictionary of model -> field names to restrict what is serialized
     fields = None
     exclude = None
     
+    # mapping of field name to resource class
+    include_resources = None
+    
     # filtering
     ignore_filters = ('ordering', 'page', 'limit', 'key', 'secret',)
     exclude_filter_fields = None
-    related_filters = []
     
     def __init__(self, rest_api, model, authentication, allowed_methods=None):
         self.api = rest_api
         self.model = model
         self.authentication = authentication
         self.allowed_methods = allowed_methods or ['GET', 'POST', 'PUT', 'DELETE']
+        
+        self._fields = {self.model: self.fields or self.model._meta.get_field_names()}
+        if self.exclude:
+            self._exclude = {self.model: self.exclude}
+        else:
+            self._exclude = {}
+        
+        self._resources = {}
+        if self.include_resources:
+            for field_name, resource in self.include_resources.items():
+                field_obj = self.model._meta.fields[field_name]
+                resource_obj = resource(self.api, field_obj.to, self.authentication, self.allowed_methods)
+                self._resources[field_name] = resource_obj
+                self._fields.update(resource_obj._fields)
+                self._exclude.update(resource_obj._exclude)
+            
+            self._include_foreign_keys = False
+        else:
+            self._include_foreign_keys = True
     
     def authorize(self):
         return self.authentication.authorize()
@@ -120,7 +143,7 @@ class RestResource(object):
         return self.model.select()
     
     def get_query_filter(self, query):
-        return QueryFilter(query, self.exclude_filter_fields, self.ignore_filters, self.related_filters)
+        return QueryFilter(query, self.exclude_filter_fields, self.ignore_filters)
     
     def get_serializer(self):
         return Serializer()
@@ -137,18 +160,18 @@ class RestResource(object):
     def serialize_object(self, obj):
         s = self.get_serializer()
         return self.prepare_data(
-            obj, s.serialize_object(obj, self.fields, self.exclude)
+            obj, s.serialize_object(obj, self._fields, self._exclude, self._include_foreign_keys)
         )
     
     def serialize_query(self, query):
         s = self.get_serializer()
         return [
-            self.prepare_data(obj, s.serialize_object(obj, self.fields, self.exclude)) \
+            self.prepare_data(obj, s.serialize_object(obj, self._fields, self._exclude, self._include_foreign_keys)) \
                 for obj in query
         ]
     
     def deserialize_object(self, data, instance):
-        return self.get_deserializer().deserialize_object(data, instance)
+        return self.get_deserializer().deserialize_object(instance, data)
     
     def response_forbidden(self):
         return Response('Forbidden', 403)
@@ -175,6 +198,7 @@ class RestResource(object):
         return (
             ('/', self.require_method(self.api_list, ['GET', 'POST'])),
             ('/<pk>/', self.require_method(self.api_detail, ['GET', 'POST', 'PUT', 'DELETE'])),
+            ('/<pk>/delete/', self.require_method(self.post_delete, ['POST', 'DELETE'])),
         )
     
     def check_get(self, obj=None):
@@ -202,20 +226,25 @@ class RestResource(object):
         elif request.method == 'POST':
             return self.create()
     
-    def api_detail(self, pk):
+    def api_detail(self, pk, method=None):
         obj = get_object_or_404(self.get_query(), **{
             self.model._meta.pk_name: pk
         })
+
+        method = method or request.method
         
-        if not getattr(self, 'check_%s' % request.method.lower())(obj):
+        if not getattr(self, 'check_%s' % method.lower())(obj):
             return self.response_forbidden()
         
-        if request.method == 'GET':
+        if method == 'GET':
             return self.object_detail(obj)
-        elif request.method in ('PUT', 'POST'):
+        elif method in ('PUT', 'POST'):
             return self.edit(obj)
-        elif request.method == 'DELETE':
+        elif method == 'DELETE':
             return self.delete(obj)
+
+    def post_delete(self, pk):
+        return self.api_detail(pk, 'DELETE')
     
     def apply_ordering(self, query):
         ordering = request.args.get('ordering') or ''
@@ -247,23 +276,6 @@ class RestResource(object):
             'next': next,
         }
     
-    def apply_filters(self, query):
-        filters = []
-        for key in request.args:
-            if key in self.ignore_filters:
-                continue
-            
-            values = request.args.getlist(key)
-            if len(values) == 1:
-                filters.append((key, values[0]))
-            else:
-                filters.append(('%s__in' % key, values))
-        
-        if filters:
-            query = query.filter(**dict(filters))
-        
-        return query
-    
     def object_list(self):
         query = self.get_query()
         query = self.apply_ordering(query)
@@ -273,7 +285,7 @@ class RestResource(object):
         
         # process the filters from the request
         filtered_query = query_filter.get_filtered_query()
-        
+
         try:
             paginate_by = int(request.args.get('limit', self.paginate_by))
         except ValueError:
@@ -302,9 +314,11 @@ class RestResource(object):
         except ValueError:
             return self.response_bad_request()
         
-        instance = self.deserialize_object(data, self.model())
+        instance, models = self.deserialize_object(data, self.model())
         instance = self.save_object(instance, data)
         
+        # TODO: save additional models
+        
         return self.response(self.serialize_object(instance))
     
     def edit(self, obj):
@@ -313,9 +327,11 @@ class RestResource(object):
         except ValueError:
             return self.response_bad_request()
         
-        obj = self.deserialize_object(data, obj)
+        obj, models = self.deserialize_object(data, obj)
         obj = self.save_object(obj, data)
         
+        # TODO: save additional models
+        
         return self.response(self.serialize_object(obj))
     
     def delete(self, obj):
@@ -364,6 +380,9 @@ class RestAPI(object):
     def unregister(self, model):
         del(self._registry[model])
     
+    def is_registered(self, model):
+        return self._registry.get(model)
+    
     def response_auth_failed(self):
         return Response('Authentication failed', 401, {
             'WWW-Authenticate': 'Basic realm="Login Required"'
